//+------------------------------------------------------------------+
//|                                                  StepMA_v7.1.mq4 |
//|                                Copyright  2006, TrendLaboratory |
//|            http://finance.groups.yahoo.com/group/TrendLaboratory |
//|                                   E-mail: igorad2003@yahoo.co.uk |
//+------------------------------------------------------------------+
//| Modifications:                                                   |
//| - StepMA_v7.2, 2006/06/26, by Ogeima, ph_bresson@yahoo.com       |
//|   - Color Mode 3                                                 |
//|   - UpDownShift to ColorMode2 and ColorMode3                     |
//|   - Fixed LineBuffer[] for every ColorMode, in order to enable   |
//|     applying a MovingAverage(1) on top of the StepMA,            | 
//|     thus enabling the charting of LevelLines on the StepMA       |
//+------------------------------------------------------------------+

#property copyright "Copyright  2006, TrendLaboratory"
#property link      "http://finance.groups.yahoo.com/group/TrendLaboratory"

#property indicator_separate_window
#property indicator_buffers 3
#property indicator_color1  Aqua
#property indicator_color2  Lime
#property indicator_color3  Red
#property indicator_width1  3
#property indicator_width2  3
#property indicator_width3  3
#property indicator_minimum 0
#property indicator_maximum 1






extern int     Length      = 10;      // Volty Length
extern double  Kv          = 0.5;     // Sensivity Factor
extern int     StepSize    = 20;       // Constant Step Size (if need)
extern int     MA_Mode     = 0;       // Volty MA Mode : 0-SMA, 1-LWMA 
extern int     Advance     = 0;       // Offset
extern double  Percentage  = 0;       // Percentage of Up/Down Moving
extern double  UpDownShift = 0;       // Number of Up/Down Moving, in points. For ColorMode==3 only.
extern bool    HighLow     = false;   // High/Low Mode Switch (more sensitive)
extern int     ColorMode   = 2;       // Color Mode Switch



   






double LineBuffer[];
double UpBuffer[];
double DnBuffer[];
double smin[];
double smax[];
double trend[];


double pointModifier=1;





  int init()
  {

   IndicatorBuffers(6);
   SetIndexBuffer(0,LineBuffer); SetIndexStyle(0,DRAW_HISTOGRAM); SetIndexShift(0,Advance);
   SetIndexBuffer(1,UpBuffer);   SetIndexStyle(1,DRAW_HISTOGRAM); SetIndexShift(1,Advance);
   SetIndexBuffer(2,DnBuffer);   SetIndexStyle(2,DRAW_HISTOGRAM); SetIndexShift(2,Advance);
   SetIndexBuffer(3,smin);
   SetIndexBuffer(4,smax);
   SetIndexBuffer(5,trend);
   

   IndicatorShortName("StepMA("+Length+","+Kv+","+StepSize+")");
   
   return(0);
   }

int deinit() { return(0); }     



int start()
{
   
   int    counted_bars = IndicatorCounted();
   int    i,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = MathMin(Bars-counted_bars,Bars-1);
         
   if (Digits==3 || Digits==5)
               pointModifier = 10;
         else  pointModifier = 1; 
         
         
         
      for(i = limit; i >= 0; i--)
      {   
     
      int Step = StepSizeCalc(Length,Kv,StepSize,i);
				
      Comment (" StepSize= ", Step);
	
   	double StepMA = StepMACalc (HighLow,Step,i) + Percentage/100.0 * Step * Point * pointModifier;
	
   	if (ColorMode == 0) LineBuffer[i] = 1;
	
   	if (ColorMode == 1)
   	{
         if (trend[i] > 0)
         {
            UpBuffer[i]  = 1;
            LineBuffer[i]= UpBuffer[i];
            DnBuffer[i]  = EMPTY_VALUE;
         }
         else
         if (trend[i] < 0)
         {
            DnBuffer[i]  = 1;
            LineBuffer[i]= DnBuffer[i];
            UpBuffer[i]  = EMPTY_VALUE;
         }
         }
         else
         if (ColorMode == 2)
         {
           if (trend[i] > 0) 
           {
             UpBuffer[i] = 1;
             if (trend[i+1] < 0) UpBuffer[i+1] = 1;
             LineBuffer[i] = UpBuffer[i];
             DnBuffer[i]   = EMPTY_VALUE;
          }
          if (trend[i] < 0) 
          {
            DnBuffer[i] = 1;
            if (trend[i+1] > 0) DnBuffer[i+1] = 1;
            LineBuffer[i] = DnBuffer[i];
            UpBuffer[i]   = EMPTY_VALUE;
         }
         }
         else
         if (ColorMode == 3)
   	   {
           if (trend[i] > 0)
           {
             UpBuffer[i]  = 1;
             LineBuffer[i]= UpBuffer[i];
             DnBuffer[i]  = EMPTY_VALUE;
           }
           else
           if (trend[i] < 0)
           {
             DnBuffer[i]  = 1;
             LineBuffer[i]= DnBuffer[i];
             UpBuffer[i]  = EMPTY_VALUE;
           }
           else
           if (trend[i] == 0)
           {	
             LineBuffer[i] = 1;
             UpBuffer[i]   = EMPTY_VALUE;
             DnBuffer[i]   = EMPTY_VALUE;
           }
      }
	
   }
	return(0);	
 }
 
  double StepSizeCalc (int Len, double Km, int Size, int k)
   {
   double ATR0=0;
   double ATRmax =-100000;
   double ATRmin=1000000;
   double result;
   
   if( Size==0 ) 
   {
        double AvgRange=0;
	     double Weight = 0;
	     for (int i=Len-1;i>=0;i--)
	     { 
        if(MA_Mode==0) double alfa= 1.0; else alfa= 1.0*(Len-i)/Len; 
           AvgRange+= alfa*(High[k+i]-Low[k+i]);
           Weight += alfa;
        }
	        ATR0 = AvgRange/Weight;
	
	if (ATR0>ATRmax) ATRmax=ATR0;
	if (ATR0<ATRmin) ATRmin=ATR0;
	
	result=MathRound(0.5*Km*(ATRmax+ATRmin)/Point*pointModifier); 
	}
	else
	result=Km*StepSize;
   
   return(result);
   }  
   
   double StepMACalc (bool HL, double Size, int k)
   {
   int counted_bars=IndicatorCounted();
   int limit;
   double result;
   
   if (HL)
	  {
	  smax[k]=Low[k]+2.0*Size*Point*pointModifier;
	  smin[k]=High[k]-2.0*Size*Point*pointModifier;
     } 	
	else  	 
	  {
	  smax[k]=Close[k]+2.0*Size*Point*pointModifier;
	  smin[k]=Close[k]-2.0*Size*Point*pointModifier;
	  }
	  if (counted_bars==0){smax[limit+1]=smax[limit];smin[limit+1]=smin[limit];trend[limit+1]=0;}
	  
	  trend[k]=trend[k+1];   
	  
	  if (Close[k]>smax[k+1]) trend[k]=1;   
	 
	  if (Close[k]<smin[k+1]) trend[k]=-1;  
	 
	  if(trend[k]>0)
	  {
	  if(smin[k]<smin[k+1]) smin[k]=smin[k+1];
	  result=smin[k]+Size*Point*pointModifier;
	  } 
	  else
	  {
	  if(smax[k]>smax[k+1]) smax[k]=smax[k+1];
	  result=smax[k]-Size*Point*pointModifier;
	  }
     return(result); 
     } 

